function [DM, MDM, DM_pval, MDM_pval] = ...
                    Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                                    dmLossFnIndex, dmTestType, dmHorizon)
  %
  %  Desc: Computes Diebold-Mariano (DM) and Modified Diebold-Mariano (MDM) 
  %        statistics for equal predictive accuracy of two forecasts of a 
  %        time series, based on Diebold-Mariano (JBES, 1995) and Harvey,
  %        Leybourne and Newbold (JBES, 1998) respectively.
  %        
  %  Parameters: Forecast1, Forecast2 - <Horizon>-step ahead forecasts of a
  %                                     time series stored in <Actual>
  %              Actual - actual time series that is being forecasted
  %              LossFnIndex - index of the function used to compute losses
  %                            associated with deviation of the forecast
  %                            from actual values of time series
  %                            0 - absolute error loss
  %                            1 - squared error loss
  %              TestType    - if H0: two forecasts have the same
  %                                   predictive accuracy,
  %                            then
  %                            "two-sided" - H1: forecasts have different
  %                                              predictive accuracy (PA)
  %                            "less"  - H1: PA(Forecast1) < PA(Forecast2)
  %                            "greater" - H1: PA(Forecast1) > PA(Forecast2)
  %             Horizon      - horizon of the forecasts 
  %                            (i.e. <horizon>-step ahead forecast)
  %  Output: DM - Diebold-Mariano statistics, MDM - Modified DM statistics
  %
  DM = 0; DM_pval = 0.0;
  MDM = 0; MDM_pval = 0.0;
  
  [obsT1, cols1] = size(dmForecast1);
  [obsT2, cols2] = size(dmForecast2);
  [obsT, cols] = size(dmActual);
  
  if not( (obsT1 == obsT2) && (obsT1 == obsT) && (cols1 == cols2) && ...
          (cols1 == cols) && (cols == 1) );
    disp('dmForecast1, dmForecast2 and dmActual should all by Mx1 vectors.');
    return;
  end
  
  if ((dmHorizon > obsT) || (dmHorizon < 1));
    disp('Invalid dmHorizon.');
    return;
  end
  
  Loss1 = zeros(obsT,1);
  Loss2 = zeros(obsT,1);
  switch dmLossFnIndex
      case 0
          Loss1 = AbsoluteLoss(dmForecast1, dmActual);
          Loss2 = AbsoluteLoss(dmForecast2, dmActual);
      case 1
          Loss1 = SquaredLoss(dmForecast1, dmActual);
          Loss2 = SquaredLoss(dmForecast2, dmActual);
      otherwise
          disp('dmLossFnIndex could be 0 (abs loss) or 1 (squared loss)');
          return;
  end 
  
  LossDiff = Loss1 - Loss2;  % Loss differentials
  CutoffIndex1 = dmHorizon - 1;
  CutoffIndex2 = 1;
  CutoffIndex = min([CutoffIndex1,CutoffIndex2]);
  
  ACF = xcov( LossDiff, 'unbiased');  % compute all autocovariances
  % 0-lag autocovariance (i.e. variance) is ACF[obsT]
  % 1-lag autocovariance  is ACF[obsT + 1] etc
  LRV = ACF(obsT); % variance first
  if (CutoffIndex > 0);
      tmp = sum(ACF(obsT+1:obsT+CutoffIndex));
      LRV = LRV + 2*sum(ACF(obsT+1:obsT+CutoffIndex));
  end    
  
  % Finally: Diebold-Mariano (DM) and Modified DM statistic
  % if LRV is negative, according to D-M we reject equal predicitive 
  % accuracy right away (treat LRV as zero)
  if LRV < 0.0;
      LRV = 1e-15;
  end  
  
  DM = mean(LossDiff)/sqrt(LRV/obsT);
  mean(LossDiff);
  sqrt(LRV/obsT);
  
  Numerator = sqrt( obsT + (1-2*dmHorizon) + dmHorizon*(dmHorizon-1)/obsT );
  MDM = (Numerator/sqrt(obsT))*DM;

  %DM = 1.96;
  %MDM = -1.96;
  
  switch lower(dmTestType)
      case 'two-sided'
          DM_pval = 2*normcdf(-abs(DM)); % cdf of standard normal distribution
          MDM_pval = 2*tcdf(-abs(MDM), obsT - 1); % cdf of Student t distribution
      case 'less'
          DM_pval = normcdf(DM); % cdf of standard normal distribution
          MDM_pval = tcdf(MDM, obsT - 1); % cdf of Student t distribution
     case 'greater'
          if (DM < 0);
            DM_pval = 1.0
          else
            DM_pval = normcdf(-abs(DM)); 
          end
          if (MDM < 0);
            MDM_pval = 1.0
          else
            MDM_pval = tcdf(-abs(MDM), obsT-1); 
          end
  end        

  
    function p = AbsoluteLoss(A, B)
        p = abs(A-B); 
    end

    function p = SquaredLoss(A, B)
        p = (A - B).^2;
    end

    function cdf = normcdf(y)
      cdf = 0.5*(1+erf(y/sqrt(2)));
      return;
    end

    function pdf = normpdf(y)
      pdf = 1/sqrt(2*pi) * exp(-1/2*y.^2);
      return;
    end


    function cdfinv = norminv(y)
      cdfinv = sqrt(2) * erfinv(2*y - 1);
      return;
    end


end