% [cf, u_var, c_var] = 
%     GetConditionalForecast_Student(actual_X1, sd_myu, 
%                                    sd_VCV, sd_s0, sd_nyu0)
% calculates conditional mean and variance of Student t distribution
%
% assumes X = [X1;X2], partition of X at 2nd element
%
% parameters:   actual_X1 - conditioning value
%               sd_myu, sd_VCV - estimates of mean and variance of X
%               sd_s0, sd_nyu0 - assumed parameters of t(myu, s, M, nyu)
%             
% note:   estimate of M is derived from s0, nyu0 and VCV
%
% returns:  cf    - conditional forecast E(X2|X1)
%           u_var - unconditional variance of X2
%           c_var - conditional variance of X2 given X1
%

function [cf, u_var, c_var] = GetConditionalForecast_Student(actual_X1, sd_myu, sd_VCV, sd_s0, sd_nyu0)
  
% suppose X ~ student_t(p) [ myu, s, M, nyu ]
%
% unconditional var-cov of X:
%
%   Var(X) = s/(nyu - 2) * M^(-1)
%
% Solve for M from given var-cov matrix of X - VCV:
% ("Matching Moments" estimator of M)
%
%   M_hat = s/(nyu - 2) * VCV^(-1)

M_hat = sd_s0/(sd_nyu0 - 2)* inv(sd_VCV);

% let X = [X1; X2]; we know X1, predict X2

div_index = 2;  % means that X is partitioned as
                %  X1 = X[1..div_index-1]
                %  X2 = X[div_index..end]

forecast_X2 = GetForecast_Student(div_index, sd_myu, sd_s0, M_hat, sd_nyu0);

condforecast_X2 = ...
  GetCondForecast_Student(div_index, actual_X1, sd_myu, sd_s0, M_hat, sd_nyu0);

var_X2 = GetVariance_Student(div_index, sd_myu, sd_s0, M_hat, sd_nyu0);

condvar_X2 = ...
  GetCondVariance_Student(div_index, actual_X1, sd_myu, sd_s0, M_hat, sd_nyu0);

cf = condforecast_X2;
u_var = var_X2;
c_var = condvar_X2;
return;

  
% ==========================================================

function ret = GetForecast_Student(div_index, sd_myu, sd_s, sd_M, sd_nyu)
  ret = NaN;
  if BadDimensions(sd_myu, sd_M);
    return;
  end
  
  myu1 = sd_myu(1:div_index-1,1);
  myu2 = sd_myu(div_index:end,1);

  ret = myu2;
end

function ret = GetCondForecast_Student(...
                 div_index, actual_X1, sd_myu, sd_s, sd_M, sd_nyu)
  ret = NaN;
  [rX1, cX1] = size(actual_X1);
  [rmyu, cmyu] = size(sd_myu);
  if BadDimensions(sd_myu, sd_M) || (cX1 ~= 1) || ...
     ((div_index + rX1 - 1) ~= rmyu)
    return;
  end

  myu1 = sd_myu(1:div_index-1,1);
  myu2 = sd_myu(div_index:end,1);
  M11 = sd_M(1:div_index-1, 1:div_index-1);
  M22 = sd_M(div_index:end, div_index:end);
  M12 = sd_M(1:div_index-1,div_index:end);
  M21 = sd_M(div_index:end, 1:div_index-1);
  
  ret = myu2 - inv(M22)*M21*(actual_X1 - myu1);
end

function ret = GetVariance_Student(div_index, sd_myu, sd_s, sd_M, sd_nyu)
  ret = NaN;
  if BadDimensions(sd_myu, sd_M);
    return;
  end
  
  M11 = sd_M(1:div_index-1, 1:div_index-1);
  M22 = sd_M(div_index:end, div_index:end);
  M12 = sd_M(1:div_index-1,div_index:end);
  M21 = sd_M(div_index:end, 1:div_index-1);
  
  M22dot1 = M22 - M21*inv(M11)*M12;
  
  ret = sd_s/(sd_nyu - 2)*inv(M22dot1);
end

function ret = GetCondVariance_Student(...
                 div_index, actual_X1, sd_myu, sd_s, sd_M, sd_nyu)
  ret = NaN;
  [rX1, cX1] = size(actual_X1);
  [rmyu, cmyu] = size(sd_myu);
  if BadDimensions(sd_myu, sd_M) || (cX1 ~= 1) || ...
     ((div_index + rX1 - 1) ~= rmyu)
    return;
  end

  myu1 = sd_myu(1:div_index-1,1);
  myu2 = sd_myu(div_index:end,1);
  M11 = sd_M(1:div_index-1, 1:div_index-1);
  M22 = sd_M(div_index:end, div_index:end);
  M12 = sd_M(1:div_index-1,div_index:end);
  M21 = sd_M(div_index:end, 1:div_index-1);

  M11dot2 = M11 - M12*inv(M22)*M21;
  q2_X1 = sd_s + (actual_X1 - myu1)'*M11dot2*(actual_X1 - myu1);
  
  ret = q2_X1/((sd_nyu+rX1)-2) * inv(M22);
end   

function ret = BadDimensions(sd_myu, sd_M)
  ret = 1;
  [nrows, ncols] = size(sd_myu);
  [nrowsM, ncolsM] = size(sd_M);
  if ( (ncols ~= 1) || (nrows < 2) || ...
       ((nrows >= 2) && (div_index > nrows)) || ...
       (div_index <= 1) );
    return;
  end
  if ( (ncolsM ~= nrowsM) || (nrows ~= nrowsM) );
    return;
  end
  ret = 0;
end    

%========================================================================  
  
  
end % GetConditionalForecast_Student