% conditional forecasting

% modified: Max Kryshko, 23-Feb-2009, Mon
% features:
%    - drawing bivariate predictive plots
%    - choice of Normal vs Student t as approximating 
%      joint predictive distribution
%

clear;

% some initialization

flag_NormalPredictiveDistribution  = 1;
flag_StudentPredictiveDistribution = 2;

flag_PredictiveDistribution = 2;

% parameters to use with Student distribution
sd_s0   = 1;
sd_nyu0 = 5;


% Load the simulation data from predDist.m (which must be run first). The
% matrix x1 is dimensioned as (lambda val, estimation horizon, forecast
% horizon, simulation, variable) the variable ordering is:
%       {y, c, i, h, w, pi, R, pce, cpce, ur}
date = '18-May-2009';
fname = strcat('preDist-',date);
load(fname,'x1');

nlam   = 3;
nestim = 27;
nsteps = 12;
nyvar  = 7;

fcastArray  = zeros(nlam,nestim+1,nsteps,nyvar);
ufcastArray = zeros(nlam,nestim+1,nsteps,nyvar);
rmseArray   = zeros(4,nlam,nyvar);
urmseArray  = zeros(4,nlam,nyvar);

% Read in data for conditioning --- in this case interest rate. Data runs
% from 2001Q1 to 2007Q4
[YY,txt,raw] = xlsread('usModelData.xls','Sheet3','a187:h214');

% Read in data on auxiliary variables to calculate rmse's
[data1,txt1,raw1] = xlsread('auxvars2.xls','Sheet2','a169:e196');
data1(:,4) = data1(:,4)/1000;

% Pad the conditioning variable data with zeros to make life easier
cvarY  = [YY(:,1);zeros(12,1)];  % output growth
cvarX1 = [YY(:,7);zeros(12,1)];  % Nominal rate
cvarX2 = [YY(:,6);zeros(12,1)];  % GDP deflator inflation

redmatn = zeros(nlam,nestim+1,nsteps,nyvar);
redmatd = zeros(nlam,nestim+1,nsteps,nyvar);

ti = 2001.0:0.25:2007.75;

fig_No = 1;
 
for ilam = 1:3
    for j = 0:nestim
        for st = 1:nsteps
                   
            mat = reshape(x1(ilam,j+1,st,:,:),180,11);
            [d1,d2] = size(mat);
            
            mu = mean(mat);
            vcv = cov(mat);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % condition on nominal interest rate (7) or infaltion (6)
            % full ordering is:
            %   y, c, i, h, w, pi, R, pce, cpce, UR,  HS
            %   1, 2, 3, 4, 5, 6,  7, 8 ,   9,   10,  11
            
            v2   = 6;
            
            %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            if v2 == 7
                xvar = cvarX1(j+st,1);
            elseif v2 == 6
                xvar = cvarX2(j+st,1);
            else
                disp('Conditioning VAR error')
            end
                       
            % Forecasts for y, h, pi, pce, cpce, ur, hs
            yvars = [1,4,6,8,9,10,11];
            
            for var = 1:nyvar;
                          
                v1 = yvars(var);

                 mu1 = mu(v1);
                 mu2 = mu(v2);
                 s12 = vcv(v1,v2);
                 s22 = vcv(v2,v2);
                 s11 = vcv(v1,v1);
                
                if (flag_PredictiveDistribution == flag_NormalPredictiveDistribution);
                  mu1 = mu(v1);
                  mu2 = mu(v2);                  
                  s12 = vcv(v1,v2);
                  s22 = vcv(v2,v2);
                  s11 = vcv(v1,v1);
            
                elseif (flag_PredictiveDistribution == flag_StudentPredictiveDistribution);
                  % we flip mu's since procedure for Student distribution
                  % computes E(X2|X1)
                  sd_mu = [mu(v2); mu(v1)];
                  sd_VCV = [ vcv(v2,v2), vcv(v2,v1); ...
                             vcv(v1,v2), vcv(v1,v1) ];
                end    
                
                if ( (v1 >= 1) && (v1 <= 7))
                  cvarY = [YY(:,v1);zeros(12,1)];  % right actual variable
                elseif ( (v1 > 7) && (v1 <= 11))
                  cvarY = [data1(:,v1-7);zeros(12,1)];  % right actual variable  
                end
                yvar = cvarY(j+st,1);
                
                %
                % generate a st-ahead forecast assuming conditional
                % normality. First period forecasted is 2001Q1
                % fcastArray(ilam,j+1,st,var) = mu1 + s12/s22 * (var2 - mu2);
                %
                if (flag_PredictiveDistribution == flag_NormalPredictiveDistribution);
                  cf   = mu1 + s12/s22 * (xvar - mu2);
                  ucf  = mu1;
                  rdsn = s11-s12*s12/s22;
                  rdsd = s11;
                elseif (flag_PredictiveDistribution == flag_StudentPredictiveDistribution);
                  % compute E(X2|X1), and theoretical SE ratio
                  [cf, u_var, c_var] = ...
                    GetConditionalForecast_Student(xvar, sd_mu, sd_VCV, sd_s0, sd_nyu0);
                  ucf  = mu1;
                  rdsn = c_var;
                  rdsd = u_var;
                end    
                
                fcastArray(ilam,j+1,st,var)  = cf;
                ufcastArray(ilam,j+1,st,var) = ucf;
                                
                %rds = sqrt( (s11-s12*s12/s22)/s11 );
                redmatn(ilam,j+1,st,var) = rdsn;
                redmatd(ilam,j+1,st,var) = rdsd;
                
                c = [ones(d1,1);2;3;4];
                s = 20*ones(d1+3,1);
                s(d1+1,1) = 100;
                s(d1+2,1) = 200;
                s(d1+3,1) = 350;
                               
                % two time periods with conditional forecast of output
                % growth getting close to actual
                
                graphit = 0;
                if graphit == 1
                    TriggerFlag = (ti(j+1) == 2006.5) || (ti(j+1) == 2001.75);

                    if (st == 1 && v2 == 7 ) && (TriggerFlag == 1)
                        if ( (var == 1) && (ilam == 3) ||    ...  % 1 - output
                             (var == 4) && (ilam == 3) || ...  % 4 - pce, lam = lam3 yields best forecast
                             (var == 6) && (ilam == 3) )     % 6 - unemployment, lam = lam2 yields best forecast

                            xvar1 = cvarX1(j+st,1);
                            if (flag_PredictiveDistribution == flag_NormalPredictiveDistribution);
                              cf = mu1 + s12/s22 * (xvar1 - mu2);
                            elseif (flag_PredictiveDistribution == flag_StudentPredictiveDistribution);
                              % compute E(X2|X1), and theoretical SE ratio
                              [cf, u_var, c_var] = ...
                                GetConditionalForecast_Student(xvar1, sd_mu, sd_VCV, sd_s0, sd_nyu0);
                            end    
                            %pause;
                            %cf = mu1 + s12/s22 * (xvar1 - mu2);

                            % adding actual observation
                            add_row1 = zeros(1, 11);
                            add_row1(v2) = xvar1;        % actual R
                            add_row1(yvars(var)) = yvar; % actual current variable
                            pmat = [mat; add_row1];

                            %pmat = [mat;[yvar,0,0,0,0,0,xvar1,0,0,0]];

                            % adding unconditional forecast
                            add_row2 = zeros(1, 11);
                            add_row2(v2) = mu2; % unconditional mean of R
                            add_row2(yvars(var)) = mu1; % unconditional forecast: current var
                            pmat = [pmat; add_row2];

                            % adding conditional forecast
                            add_row3 = zeros(1, 11);
                            add_row3(v2) = xvar1; % actual R
                            add_row3(yvars(var)) = cf; % conditional forecast: current var
                            pmat = [pmat; add_row3];

                            %pmat = [pmat;[mu1,0,0,0,0,0,mu2,0,0,0];[cf,0,0,0,0,0,xvar1,0,0,0]];

                            %figure(fig_No);
                            %scatter(pmat(:,7),pmat(:,1),s,c,'filled');
                            scatter(pmat(:,v2),pmat(:,yvars(var)),s,c,'filled');
                            % ti is constructed this way (real number):
                            % 2000:Q1 = 2000.0
                            % 2000:Q2 = 2000.25
                            % 2000:Q3 = 2000.50
                            % 2000:Q4 = 2000.75
                            % next line extracts quarter number
                            Quarter_No = ( ti(j+1) - floor(ti(j+1)) )*4 + 1;
                            Year_No = floor(ti(j+1)); 
                            chrRomanQNumbers = ['I  '; 'II '; 'III'; 'IV '];
                            RomanQNumbers = cellstr(chrRomanQNumbers);
                            ts = strcat(num2str(Year_No),':',RomanQNumbers(Quarter_No));
                            % ts = strcat('1-step ahead conditional forecast for: ',num2str(ti(j+1)));
                            title(ts, 'FontSize', 24);
                            xlabel('Interest Rate (A %)', 'FontSize', 20);
                            YAxisName = '';
                            YAxisVarTag = '';
                            switch var
                                case 1
                                    YAxisName = 'Output Growth (Q %)';
                                    YAxisVarTag = 'Y';
                                    axis([0,8,-3,3]);
                                case 4
                                    YAxisName = 'PCE Inflation (Q %)';
                                    YAxisVarTag = 'PCE-';
                                    axis([0,8,-1,1.5]);
                                case 6
                                    YAxisName = 'Unemployment Rate (%)';
                                    YAxisVarTag = 'UR-';
                                    axis([0,8,4,6]);
                                otherwise
                                    axis([0,8,-3,3]);
                            end
                            set(get(gcf, 'CurrentAxes'),'FontSize',20);
                            ylabel(YAxisName, 'FontSize', 20);
                            %axis([0,8,-3,3]);
                            fig_No = fig_No + 1;  
                            FigureFileNamePath = 'figures_cond/';
                            mkdir(FigureFileNamePath);
                            switch ti(j+1)
                                case 2001.75
                                    TimePeriodTag = 't1';
                                case 2006.5
                                    TimePeriodTag = 't2';
                                otherwise
                                    TimePeriodTag = 't';
                            end
                            FileNameTemp = strcat('fig',YAxisVarTag,'R_h1_',TimePeriodTag);
                            figFileNameTemp = strcat(FileNameTemp,'.fig');
                            pdfFileNameTemp = strcat(FileNameTemp,'.pdf');
                            FigureFileName = strcat(FigureFileNamePath,figFileNameTemp);
                            saveas(gcf, FigureFileName);
                            FigureFileName = strcat(FigureFileNamePath,pdfFileNameTemp);
                            saveas(gcf, FigureFileName);
                            pause; 
                        end
                    end
                end
                
                if graphit == 1
                    if (st == 1 && v2 == 6)
                        if var == 1

                            xvar2 = cvarX2(j+st,1);
                            if (flag_PredictiveDistribution == flag_NormalPredictiveDistribution);
                              cf = mu1 + s12/s22 * (xvar2 - mu2);
                            elseif (flag_PredictiveDistribution == flag_StudentPredictiveDistribution);
                              % compute E(X2|X1), and theoretical SE ratio
                              [cf, u_var, c_var] = ...
                                GetConditionalForecast_Student(xvar2, sd_mu, sd_VCV, sd_s0, sd_nyu0);
                            end    

                            %cf = mu1 + s12/s22 * (xvar2 - mu2);

                            pmat = [mat;[yvar,0,0,0,0,xvar2,0,0,0,0,0]];
                            pmat = [pmat;[mu1,0,0,0,0,mu2,0,0,0,0,0];[cf,0,0,0,0,xvar2,0,0,0,0,0]];

                            scatter(pmat(:,6),pmat(:,1),s,c,'filled');
                            ts = strcat('1-step ahead predictive for: ',num2str(ti(j+1)));
                            title(ts);
                            xlabel('{\Pi}');
                            ylabel('{\Delta y}');
                            axis([-1,2,-3,3]);
                            %pause; 
                        end
                    end
                end
                               
            end  
        end
        
        
    end
end

%% Set up actuals data
% Column ordering is y, c, i, h, w, pi, R, pce, cpce, ur

% 1-step from 2000Q4+j,  j = 0,1,2,...
sa1  = xlsread('auxvars2.xls','Sheet2','a169:e196');
sa1  = [YY(1:end,:),sa1];

% 2-step from 2000Q4+j, j = 0,1,2,...
sa2  = xlsread('auxvars2.xls','Sheet2','a170:e196');
sa2 = [YY(2:end,:),sa2];

% 4-step from 2000Q4+j, j = 0,1,2,...
sa4  = xlsread('auxvars2.xls','Sheet2','a172:e196');
sa4 = [YY(4:end,:),sa4];

% 12-step from 2000Q4+j, j = 0,1,2,...
sa12 = xlsread('auxvars2.xls','Sheet2','a180:e196');
sa12 = [YY(12:end,:),sa12];

% Trim the actuals data to pull out y, h, pi, pce, cpce, ur, hs
vlist = [1,4,6,8,9,10,11];
sa1   = sa1(:,vlist);
sa2   = sa2(:,vlist);
sa4   = sa4(:,vlist);
sa12  = sa12(:,vlist);

%%

for ilam = 1:nlam
    for var = 1:nyvar

        s1  = fcastArray(ilam,1:end,1,var)'     -  sa1(:,var);
        s2  = fcastArray(ilam,1:end-1,2,var)'   -  sa2(:,var);
        s4  = fcastArray(ilam,1:end-3,4,var)'   -  sa4(:,var);
        s12 = fcastArray(ilam,1:end-11,12,var)' -  sa12(:,var);

        rmse1  = sqrt( s1'*s1/max(size(s1)) );
        rmse2  = sqrt(s2'*s2/max(size(s2)));
        rmse4  = sqrt(s4'*s4/max(size(s4)));
        rmse12 = sqrt(s12'*s12/max(size(s12)));
        
        rmseArray(1,ilam,var) = rmse1;
        rmseArray(2,ilam,var) = rmse2;
        rmseArray(3,ilam,var) = rmse4;
        rmseArray(4,ilam,var) = rmse12;
        
    end
end

for ilam = 1:nlam
    for var = 1:nyvar

        us1  = ufcastArray(ilam,1:end,1,var)'     -  sa1(:,var);
        us2  = ufcastArray(ilam,1:end-1,2,var)'   -  sa2(:,var);
        us4  = ufcastArray(ilam,1:end-3,4,var)'   -  sa4(:,var);
        us12 = ufcastArray(ilam,1:end-11,12,var)' -  sa12(:,var);

        urmse1  = sqrt( us1'*us1/max(size(us1)) );
        urmse2  = sqrt(us2'*us2/max(size(us2)));
        urmse4  = sqrt(us4'*us4/max(size(us4)));
        urmse12 = sqrt(us12'*us12/max(size(us12)));
        
        urmseArray(1,ilam,var) = urmse1;
        urmseArray(2,ilam,var) = urmse2;
        urmseArray(3,ilam,var) = urmse4;
        urmseArray(4,ilam,var) = urmse12;
        
    end
end



%%

dsge_rmse = ...
[   rmseArray(1,1,1),rmseArray(2,1,1),rmseArray(3,1,1),rmseArray(4,1,1);
    100*rmseArray(1,1,2),100*rmseArray(2,1,2),100*rmseArray(3,1,2),100*rmseArray(4,1,2);
    rmseArray(1,1,3),rmseArray(2,1,3),rmseArray(3,1,3),rmseArray(4,1,3)];

udsge_rmse = ...
[   urmseArray(1,1,1),urmseArray(2,1,1),urmseArray(3,1,1),urmseArray(4,1,1);
    100*urmseArray(1,1,2),100*urmseArray(2,1,2),100*urmseArray(3,1,2),100*urmseArray(4,1,2);
    urmseArray(1,1,3),urmseArray(2,1,3),urmseArray(3,1,3),urmseArray(4,1,3)];


pce_rmse = ...
 [  rmseArray(1,1,4),rmseArray(2,1,4),rmseArray(3,1,4),rmseArray(4,1,4);
    rmseArray(1,2,4),rmseArray(2,2,4),rmseArray(3,2,4),rmseArray(4,2,4);
    rmseArray(1,3,4),rmseArray(2,3,4),rmseArray(3,3,4),rmseArray(4,3,4)];

upce_rmse = ...
 [  urmseArray(1,1,4),urmseArray(2,1,4),urmseArray(3,1,4),urmseArray(4,1,4);
    urmseArray(1,2,4),urmseArray(2,2,4),urmseArray(3,2,4),urmseArray(4,2,4);
    urmseArray(1,3,4),urmseArray(2,3,4),urmseArray(3,3,4),urmseArray(4,3,4)];

cpce_rmse = ...
 [  rmseArray(1,1,5),rmseArray(2,1,5),rmseArray(3,1,5),rmseArray(4,1,5);
    rmseArray(1,2,5),rmseArray(2,2,5),rmseArray(3,2,5),rmseArray(4,2,5);
    rmseArray(1,3,5),rmseArray(2,3,5),rmseArray(3,3,5),rmseArray(4,3,5)];

ucpce_rmse = ...
 [  urmseArray(1,1,5),urmseArray(2,1,5),urmseArray(3,1,5),urmseArray(4,1,5);
    urmseArray(1,2,5),urmseArray(2,2,5),urmseArray(3,2,5),urmseArray(4,2,5);
    urmseArray(1,3,5),urmseArray(2,3,5),urmseArray(3,3,5),urmseArray(4,3,5)];


ur_rmse = ...
 [  rmseArray(1,1,6),rmseArray(2,1,6),rmseArray(3,1,6),rmseArray(4,1,6);
    rmseArray(1,2,6),rmseArray(2,2,6),rmseArray(3,2,6),rmseArray(4,2,6);
    rmseArray(1,3,6),rmseArray(2,3,6),rmseArray(3,3,6),rmseArray(4,3,6)];

uur_rmse = ...
 [  urmseArray(1,1,6),urmseArray(2,1,6),urmseArray(3,1,6),urmseArray(4,1,6);
    urmseArray(1,2,6),urmseArray(2,2,6),urmseArray(3,2,6),urmseArray(4,2,6);
    urmseArray(1,3,6),urmseArray(2,3,6),urmseArray(3,3,6),urmseArray(4,3,6)];


hs_rmse = ...
  [ rmseArray(1,1,7),rmseArray(2,1,7),rmseArray(3,1,7),rmseArray(4,1,7);
    rmseArray(1,2,7),rmseArray(2,2,7),rmseArray(3,2,7),rmseArray(4,2,7);
    rmseArray(1,3,7),rmseArray(2,3,7),rmseArray(3,3,7),rmseArray(4,3,7)]; 

uhs_rmse = ...
  [ urmseArray(1,1,7),urmseArray(2,1,7),urmseArray(3,1,7),urmseArray(4,1,7);
    urmseArray(1,2,7),urmseArray(2,2,7),urmseArray(3,2,7),urmseArray(4,2,7);
    urmseArray(1,3,7),urmseArray(2,3,7),urmseArray(3,3,7),urmseArray(4,3,7)];  

%% Reduction results

% need to drop "wild" periods: last 11 with h=12, 3 with h=4, 1 with h=2,
% 0 with h = 1. like in fcastArray above
lastT = nestim+1;
sel_h1 = [1:lastT];
sel_h2 = [1:lastT-1];
sel_h4 = [1:lastT-3];
sel_h12 = [1:lastT-11];

% output
yn1  = mean(redmatn(1,sel_h1,1,1));
yn2  = mean(redmatn(1,sel_h2,2,1));
yn4  = mean(redmatn(1,sel_h4,4,1));
yn12 = mean(redmatn(1,sel_h12,12,1));

yd1  = mean(redmatd(1,sel_h1,1,1));
yd2  = mean(redmatd(1,sel_h2,2,1));
yd4  = mean(redmatd(1,sel_h4,4,1));
yd12 = mean(redmatd(1,sel_h12,12,1));

y1  = sqrt(yn1/yd1);
y2  = sqrt(yn2/yd2);
y4  = sqrt(yn4/yd4);
y12 = sqrt(yn12/yd12);

% hours
hn1  = mean(redmatn(1,sel_h1,1,2));
hn2  = mean(redmatn(1,sel_h2,2,2));
hn4  = mean(redmatn(1,sel_h4,4,2));
hn12 = mean(redmatn(1,sel_h12,12,2));

hd1  = mean(redmatd(1,sel_h1,1,2));
hd2  = mean(redmatd(1,sel_h2,2,2));
hd4  = mean(redmatd(1,sel_h4,4,2));
hd12 = mean(redmatd(1,sel_h12,12,2));

h1  = sqrt(hn1/hd1);
h2  = sqrt(hn2/hd2);
h4  = sqrt(hn4/hd4);
h12 = sqrt(hn12/hd12);

% GDP deflator inflation
pn1  = mean(redmatn(1,sel_h1,1,3));
pn2  = mean(redmatn(1,sel_h2,2,3));
pn4  = mean(redmatn(1,sel_h4,4,3));
pn12 = mean(redmatn(1,sel_h12,12,3));

pd1  = mean(redmatd(1,sel_h1,1,3));
pd2  = mean(redmatd(1,sel_h2,2,3));
pd4  = mean(redmatd(1,sel_h4,4,3));
pd12 = mean(redmatd(1,sel_h12,12,3));

p1  = sqrt(pn1/pd1);
p2  = sqrt(pn2/pd2);
p4  = sqrt(pn4/pd4);
p12 = sqrt(pn12/pd12);


% DSGE vars

m1 = ...
[y1, y2, y4, y12 ;
 h1, h2, h4, h12 ;
 p1, p2, p4, p12 ];

%% PCE

m2 = theory_reduction(redmatn,redmatd,4,nestim);

%% CPCE
m3 = theory_reduction(redmatn,redmatd,5,nestim);

%% UR
m4 = theory_reduction(redmatn,redmatd,6,nestim);

%% HS    
m5 = theory_reduction(redmatn,redmatd,7,nestim);

%%
fileid = 'ijfFinal';

if v2 ==7 && flag_PredictiveDistribution == 1;
    fname = strcat(fileid,'normal_condFcst_out_R');
    save(fname,'dsge_rmse','udsge_rmse','pce_rmse','upce_rmse','cpce_rmse','ucpce_rmse',...
        'ur_rmse','uur_rmse','hs_rmse','uhs_rmse','m1','m2','m3','m4','m5');
end

if v2 == 6 && flag_PredictiveDistribution == 1;
    fname = strcat(fileid,'normal_condFcst_out_pi');
    save(fname,'dsge_rmse','udsge_rmse','pce_rmse','upce_rmse','cpce_rmse','ucpce_rmse',...
        'ur_rmse','uur_rmse','hs_rmse','uhs_rmse','m1','m2','m3','m4','m5');
end

if v2 ==7 && flag_PredictiveDistribution == 2;
    fname = strcat(fileid,'t_condFcst_out_R');
    save(fname,'dsge_rmse','udsge_rmse','pce_rmse','upce_rmse','cpce_rmse','ucpce_rmse',...
        'ur_rmse','uur_rmse','hs_rmse','uhs_rmse','m1','m2','m3','m4','m5');
end

if v2 == 6 && flag_PredictiveDistribution == 2;
    fname = strcat(fileid,'t_condFcst_out_pi');
    save(fname,'dsge_rmse','udsge_rmse','pce_rmse','upce_rmse','cpce_rmse','ucpce_rmse',...
        'ur_rmse','uur_rmse','hs_rmse','uhs_rmse','m1','m2','m3','m4','m5');
end






