% Program to calculate impulse responses from estimated DSGE model
%
% Keith Sill
% January 2008
%

% Read in the parameter chain
%
% load parameter values from posterior simulator and calculate mean
% nblocks is the number of blocks to be read out of 100. Thus, 100-nblocks
% is the "burn-in"

loaddate = 'output-20-Jun-2008/';

mean_mat = zeros(70,40);

nblocks = 70;
nsim    = 1000;
npara   = 40;

paramat = zeros(nblocks*nsim,npara);
trej = 0;

j = 24;

for i = 1:nblocks
    
    clear fname parasim;

    fname = strcat(loaddate,'smpl-1984Q1+',num2str(67+j),'--block-',num2str(i+(100-nblocks)));
    load(fname,'parasim','rej');
    ridx1 = (i-1)*nsim + 1;
    ridx2 = i*nsim;
    paramat(ridx1:ridx2,:) = parasim;
    trej = sum(rej) + trej;
end

% mean and standard deviation of draws
mpara = mean(paramat);
spara = std(paramat);
rej_pct = trej/(nblocks*nsim);

% State space representation at estimated parameter vector
%       S(t) = TT*S(t-1) + RR*e(t)
%       e(t) ~ iid N(0,QQ)
%       y(t) = DD + ZZ*x(t) + u(t)
%       u(t) ~ N(0,HH)
%  
%       cov(e(t),u(t)) = VV
%
% The system shock vector e(t) is ordered as:
%   [ z   ;
%     phi ;
%     mu  ;
%     b   ;
%     g   ;
%     laf ;
%     R   ]
%
% The observation vector y(t) is:
%   [  real per-capita GDP growth quarterly percent;
%      real per-capita consumption growth quarterly percent;
%      real per-capita investment growth quarterly percent;
%      log hours worked per capita quarterly;
%      log hourly real wage;
%      GDP deflator inflation annualized percent;
%      nominal fed funds rate annualized percent  ]

[TT,QQ,RR,HH,DD,ZZ,VV,RC]=sysmat(mpara);

% Form the reduced-state selection matrix
%
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed as 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;


% Initialize impulse response and shock matricies
[rs,cs] = size(TT);
[rr,cr] = size(RR);
[rm,cm] = size(ZZ);



%% Response to interest rate shock

% Generate the responses
% nsteps is the number of steps
nsteps = 12;
imat = zeros(nsteps,rs);
cimat = zeros(nsteps,rs);
emat = zeros(nsteps,cr);

% initialize the shock vector here
emat(2,7) = sqrt(QQ(7,7));

Romat = [];
cRomat = [];

% first get the non-cumulative responses
for t = 2:nsteps
    
    m0 = TT * imat(t-1,:)' + RR * emat(t,:)';
    imat(t,:) = m0';
    y0 = ZZ*imat(t,:)';
    
    Romat(t,:) = y0';
   
end

% now get the cumulative responses

cy0 = zeros(7,1);
cimat = zeros(nsteps,rs);
emat = zeros(nsteps,cr);
emat(2,7) = sqrt(QQ(7,7));

for t = 2:nsteps
    
    cm0 = TT * cimat(t-1,:)' + RR * emat(t,:)';
    cimat(t,:) = cm0';
    cy0 = ZZ*cimat(t,:)' + cy0;
    
    cRomat(t,:) = cy0';
   
end    

figure('Name','Response to R shock');
subplot(3,1,1);
plot(Romat(2:nsteps,1));
title('output growth');

subplot(3,1,2);
plot(Romat(2:nsteps,6));
title('inflation');

subplot(3,1,3);
plot(Romat(2:nsteps,7));
title('nominal rate');

figure('Name','Cumulative Responses to R shock');
subplot(3,1,1);
plot(cRomat(2:nsteps,1));
title('output growth');

subplot(3,1,2);
plot(cRomat(2:nsteps,6));
title('inflation');

subplot(3,1,3);
plot(cRomat(2:nsteps,7));
title('nominal rate');

%%
figure('Name','Figure 3, panel A');
subplot(3,1,1);
plot(cRomat(2:nsteps,1));
title('Y');

subplot(3,1,2);
plot(Romat(2:nsteps,6));
title('{\pi^{GDP}}');

subplot(3,1,3);
plot(Romat(2:nsteps,7));
title('R');

%% Generate impulses for auxiliary variables
nsteps = 12;

imat = zeros(nsteps,rs);
ipce = zeros(nsteps,3);

fname = 'output-20-Jun-2008/pvec_pce-lambda-1';
load(fname,'pvec_pce');
pv1 = pvec_pce;

fname = 'output-20-Jun-2008/pvec_pce-lambda-2';
load(fname,'pvec_pce');
pv2 = pvec_pce;

fname = 'output-20-Jun-2008/pvec_pce-lambda-3';
load(fname,'pvec_pce');
pv3 = pvec_pce;

for t = 2:nsteps
    
    m0 = TT * imat(t-1,:)' + RR * emat(t,:)';
    imat(t,:) = m0';
    
    ipce(t,1) = m0' * M * pv1(2:12)';
    ipce(t,2) = m0' * M * pv2(2:12)';
    ipce(t,3) = m0' * M * pv3(2:12)';
end

imat = zeros(nsteps,rs);
icpce = zeros(nsteps,3);

fname = 'output-20-Jun-2008/pvec_cpce-lambda-1';
load(fname,'pvec_cpce');
pv1 = pvec_cpce;

fname = 'output-20-Jun-2008/pvec_cpce-lambda-2';
load(fname,'pvec_cpce');
pv2 = pvec_cpce;

fname = 'output-20-Jun-2008/pvec_cpce-lambda-3';
load(fname,'pvec_cpce');
pv3 = pvec_cpce;

for t = 2:nsteps
    
    m0 = TT * imat(t-1,:)' + RR * emat(t,:)';
    imat(t,:) = m0';
    
    icpce(t,1) = m0' * M * pv1(2:12)';
    icpce(t,2) = m0' * M * pv2(2:12)';
    icpce(t,3) = m0' * M * pv3(2:12)';
end

imat = zeros(nsteps,rs);
iur  = zeros(nsteps,3);

fname = 'output-20-Jun-2008/pvec_ur-lambda-1';
load(fname,'pvec_ur');
pv1 = pvec_ur;

fname = 'output-20-Jun-2008/pvec_ur-lambda-2';
load(fname,'pvec_ur');
pv2 = pvec_ur;

fname = 'output-20-Jun-2008/pvec_ur-lambda-3';
load(fname,'pvec_ur');
pv3 = pvec_ur;

cur1 = 0;
cur2 = 0;
cur3 = 0;

for t = 2:nsteps
    
    m0 = TT * imat(t-1,:)' + RR * emat(t,:)';
    imat(t,:) = m0';
    
%     cur1 = cur1 + m0' * M * pv1(2:12)';
%     cur2 = cur2 + m0' * M * pv2(2:12)';
%     cur3 = cur3 + m0' * M * pv3(2:12)';
%     
%     iur(t,1) = cur1;
%     iur(t,2) = cur2;
%     iur(t,3) = cur3;
%     
    iur(t,1) = m0' * M * pv1(2:12)';
    iur(t,2) = m0' * M * pv2(2:12)';
    iur(t,3) = m0' * M * pv3(2:12)';
end

figure('Name','Figure 3 Panel B');

xi = (1:1:11)';
subplot(3,1,1);
plot(xi,ipce(2:end,1),'b--',xi,ipce(2:end,3),'r');
title('{\pi^{PCE}}');

subplot(3,1,2);
plot(xi,icpce(2:end,1),'b--',xi,icpce(2:end,3),'r');
%legend('1','2','3');
title('{\pi^{CPCE}}');

subplot(3,1,3);
plot(xi,iur(2:end,1),'b--',xi,iur(2:end,3),'r');
%legend('1','2','3');
title('ur');
%legend('{\lambda=1}','{\lambda=0.1}','{\lambda=1e-5}','Location','BestOutside');
%legend boxoff;
