
function [mlh,n1,n2,n3,n4,d1,d2,d3] = margLambdaV2(z,states,ahat,rhohat,sighat2,ua0,Va0,v,s,bhat,niter,burnin,tau)

    % Subroutine to compute marginal likelihood of lambda given the data
    %
    % Inputs:
    %   z:          auxiliary variable data series
    %   states:     filtered state estimates
    %   ahat:       mean coefficient vector from Gibbs sampler output
    %   rhohat:     mean rho from Gibbs sampler
    %   sighat2:    mean variance from Gibbs sampler
    %   ua0:        prior mean for alpha
    %   Va0:        prior vcv for alpha
    %   v:          inverse gamma parameter
    %   s:          inverse gamma parameter
    %   bhat:       {alpha,rho,sigma} draws from Gibbs sampler
    %   niter:      number of iterations for reduced Gibbs sampler run
    %   burnin:     burnin for reduced Gibbs sampler
    %   tau:        adjustment factor
    %
    % Output:
    %   mlh:        marginal likelihood of lambda
    %
    %   Keith Sill
    %   February 2009
    %
    
    [T,ns] = size(states);
    k      = max(size(ahat));

    %z(1) = ahat(1) + states(1,:)*ahat(2:12)';
    
    
    % NUMERATOR TERMS
    
    % ++++++++++++++ p(Y|ahat,rhohat,sighat2) +++++++++++++++++++++++
    resids = z(2:T) - rhohat .* z(1:T-1) - ...
        ahat(1) * (1-rhohat) .* ones(T-1,1) - ...
        ( states(2:T,:) - states(1:T-1,:) .* rhohat ) * ahat(2:12)';
    
    %resids(1) = z(1)*sqrt(sighat2)/tau;
    resids = [resids; (z(1)-ahat(1) - states(1,:)*ahat(2:12)')*sqrt(sighat2)/tau];
    
    n1 = (2*pi*sighat2)^(-T/2)*exp(-1/(2*sighat2)*resids'*resids);
    n1 = log(n1);
    
    % ++++++++++++++++++ p(ahat) +++++++++++++++++++++++++++++++++++++++
    n2 = (2*pi)^(-k/2)*det(Va0)^(-1/2)* exp(-1/2*(ahat'-ua0)'*inv(Va0)*(ahat'-ua0));
    n2 = log(n2);
    
    % +++++++++++++++++++ p(rhohat) ++++++++++++++++++++++++++++++++++++
    if abs(rhohat) < 1
        n3 = 1;
        n3 = log(n3);
    else
        n3 = -1e20;
    end
    
    % ++++++++++++++++++++ p(sighat2) ++++++++++++++++++++++++++++++++++
    
    
    n4 = -v/2*log(2) - gammaln(v/2) + v/2*log(v*s^2) ...
        -(v+2)/2*log(sighat2) - (v*s^2)/(2*sighat2);

    % ++++++++++++++++++++ numerator is: +++++++++++++++++++++++++++++++
    %numer = n1 + n2 + n3 + n4;
    
    % DENOMINATOR TERMS

    % ++++++++++++++++ p(ahat|rhoat,sighat2,Y) +++++++++++++++++++++++++
    ymat = z(2:T) - rhohat.*z(1:T-1);
    xmat = [(1-rhohat).*ones(T-1,1),states(2:T,:)-states(1:T-1,:).*rhohat];
    
    ymat(1)   = sqrt(sighat2)/tau * z(1);
    xmat(1,:) = sqrt(sighat2)/tau * [1,states(1,:)];
    
    alphat = inv(xmat'*xmat)*xmat'*ymat;

    VaT = inv(inv(Va0) + 1/sighat2 * xmat'*xmat);
    uaT = VaT*(inv(Va0)*ua0 + 1/sighat2 * xmat'*xmat*alphat);
   
    d1 = (2*pi)^(-k/2)*det(VaT)^(-1/2)*exp(-1/2 * (ahat'-uaT)'*inv(VaT)*(ahat'-uaT));
    d1 = log(d1);

    % +++++++++++++++++ p(sighat2|Y) +++++++++++++++++++++++++++++++++++
    [r,c] = size(bhat);
    ds = zeros(r,1);
    
    % integrate out rho(i) and alpha(i). Here, bhat is the output from the
    % full Gibbs sampler run
    for i = 1:r
        rhos = bhat(i,13);
        ahs  = bhat(i,1:12);
        
        eta = z(2:T) - ( rhos.*z(1:T-1) + ahs(1)*(1-rhos).*ones(T-1,1) ...
        + (states(2:T,:) - states(1:T-1,:).*rhos)*ahs(2:12)');

        w2 = eta'*eta;
        Tn = T-1;
        %ds(i) = -gammaln((T-1)/2) + (T-1)/2*log(s2/2) - (T+1)/2*log(sighat2) - s2/(2*sighat2);
        ds(i) = -(Tn+v)/2*log(2) - gammaln((Tn+v)/2) + (Tn+v)/2*log(w2) ...
            -(Tn+v+2)/2*log(sighat2) - w2/(2*sighat2) ;
    end
    
    dsimax = max(ds);
    ds     = ds - dsimax;
    d2     = log(mean(exp(ds))) + dsimax;
    
    %++++++++++++++++++++ p(rhohat|sighat2,Y) ++++++++++++++++++++++++++
    % reduced Gibbs Sampler run

    rbhat  = reduced_gibbs_lsq(niter,z,states,ua0,Va0,sighat2,tau);
    rbhat  = rbhat(burnin:end,:);
    [r,c] = size(rbhat);
    prhat = [];

    for i = 1:r

            avec = rbhat(i,1:12);
            y   = z(2:T) - avec(1).*ones(T-1,1) - states(2:T,:)*avec(2:12)';
            x   = z(1:T-1) - avec(1).*ones(T-1,1) - states(1:T-1,:)*avec(2:12)';
            
            urT = inv(x'*x)*x'*y;
            VrT = sighat2.*inv(x'*x);
            
            tmat(i,:) = [urT,VrT];

            prhatv = normpdf(rhohat,urT,sqrt(VrT))/(normcdf(1,urT,sqrt(VrT))-normcdf(-1,urT,sqrt(VrT)));
            j = 0;
            if isnan(prhatv)
                j = j+1;
                continue
            end
            
            prhat(i) = prhatv;
            
    end
    
%     min(log(prhat))
%     max(log(prhat))
    %frac = j
    %disp(sprintf('%7.4f',frac))
    
    d3 = mean(prhat);
    d3 = log(d3);

    % ++++++++++++++++++++ marginal likelihood ++++++++++++++++++++++++++
    mlh = n1+n2+n3+n4-d1-d2-d3;
    
%     n1
%     n2
%     n3
%     n4
%     d1
%     d2
%     d3
    

end

function omat = reduced_gibbs_lsq(niter,data,states,mu0,v0,sig0,tau)

%  Reduced Gibbs sampler --- sighat2 is fixed.

    [T,ny] = size(data);
    [nvars,nstates] = size(states);
    
    omat = zeros(niter,nstates+2);
    
    alp = mu0 + chol(v0)'*randn(nstates+1,1);
    rho = rand(1);
    sig2 = sig0;
    
    v0i = inv(v0);
    
    for i = 1:niter
        
        % find the conditional posterior of alpha given rho and sigeta
        sig2s = sqrt(sig2);
        y = [data(2:T,:) - rho*data(1:T-1,:);(data(1,:)*sig2s/tau)];
        
        x = [ones(T-1,1).*(1-rho),(states(2:T,:)-rho.*states(1:T-1,:));
            sig2s/tau, sig2s/tau * states(1,:)];
        
        xtx = x'*x;
        xty = x'*y;
        ixtx = inv(xtx);
        alphat = ixtx*xty;
        
        vat = inv(v0i + 1/sig2 * xtx);
        muat = vat*(v0i*mu0 + 1/sig2 * xtx*alphat);
        alp = muat + chol(vat)'*randn(nstates+1,1);
        
        ssqr = (y - x*alphat)'*(y-x*alphat);
        
        %if sqrt(ssqr) > eps
        
            % find the conditional posterior of rho given alpha and sigeta
        
            y = data(2:T,:) - ([ones(T-1,1),states(2:T,:)])*alp;
            x = data(1:T-1,:) - ([ones(T-1,1),states(1:T-1,:)]*alp);
            
            xtx = x'*x;
      
            xty = x'*y;
            ixtx = inv(xtx);
            mur = ixtx*xty;
           
            vrt = sig2*ixtx;
            %vrtsq = chol(vrt)';
                    
            rho = normt_rnd(mur, vrt, -1, 1 );
            
         %  if rho == -Inf
         %    rho = -1 + eps;
         %  end

        %else
            
         %   rho = 0.0;
            
        %end
                
              
%         % find the conditional posterior of sigeta given alpha and rho
%         
%         eta = data(2:T,:) - (rho*data(1:T-1,:)+alp(1)*(1-rho)+...
%             (states(2:T,:)-rho*states(1:T-1,:))*alp(2:nstates+1));
%         
%         sesq = eta'*eta;
%         z = sqrt(1/sesq)*randn(T-1,1);
%         sig2 = 1/(z'*z);

        
        omat(i,:) = [alp',rho];
        
        
    end
        
end

%////////////////////////////////////////////////////////////////////
%//
%// Random draw from truncated normal distribution
%//
%////////////////////////////////////////////////////////////////////

function fx = normt_rnd(mu,sigma2,left,right)


    std = sqrt(sigma2);

    nobs = max(size(mu));
    
    %// calculate bounds on probabilities
    lowerProb = Phi((left-mu)./std);
    upperProb = Phi((right-mu)./std);

    %// Draw uniform from within (lowerProb,upperProb)
    u = lowerProb + (upperProb-lowerProb).*rand(nobs,1);

    %// Find quantiles
    fx = mu + Phiinv(u).*std;

end 

%////////////////////////////////////////////////////////
%//
%// compute standard normal quantile function of the vector x (0,1)
%//
%////////////////////////////////////////////////////////

function fx = Phiinv(x)

    fx = sqrt(2)*erfinv(2*x-1);
end

%//////////////////////////////////////////////////////////
%//
%// standard normal distribution function at x
%//
%/////////////////////////////////////////////////////////

function fx = Phi(x)

    fx = 0.5*(1+erf(x/sqrt(2)));

end

