function [smat,DD,ZZ,retcode] = state_fcst2(para,YY,steps)
%    
% Generate model state variable sequence
% 
% Given a posterior draw para, draw a random seqence {e(T+1),...e(T+h)} and
% iterate the state transition matrix forward to construct
%   S(T+h|T) = TT(para) * S(T+h-1|T) + RR(para) * e(T+h)  for h = 1,2,...
%
% The initial state S(T|T) is calculated from the Kalman filter and T
% represents the date of the last row of the data vector YY
%
% Inputs:
%           para:       model parameter vector
%           YY:         T x n data matrix. Note that the last period of YY
%                       represents the T from which the state generation
%                       sequence begins.
%           steps:      steps ahead to simulate
%
% Outputs:
%           smat:       (nstates x nsteps) matrix with rows corresponding
%                       to model state variables and columns corresponding 
%                       forward iterations beginning at 1 ahead from T
%           retcode:    Model solution indicator from function sysmat
%
% This program calls:
%                       sysmat
%                       dlyap
%
% Keith Sill
% July 2008
%

[nobs,ny] = size(YY);

% /**********************************************************
% **      System matrices for State-space
% **********************************************************/
[TT,QQ,RR,HH,DD,ZZ,VV,RC]=sysmat(para);

% /**********************************************************
% ** Check determinacy 
% **********************************************************/
if (RC(1) == 1) && (RC(2)==1);
   % determinacy 
   retcode(1) = 0;

elseif (RC(1) == 1) && (RC(2)==0) 
   % indeterminacy 
   retcode(1) = 1;
   return;

else
   % no equilibrium exists, numerical problems 
   retcode(1) = RC(1);
   return;

end

% /**********************************************************
% **      Kalman filter generates state matrix
% **********************************************************/
[nstate,nfs] = size(TT);

% /** initial mean and variance for the state vector **/
At = zeros(nstate,1);
Pt = dlyap(TT,RR*QQ*RR');

[ns,neps] = size(RR);

%/** filter loop **/

for t = 1:nobs

   At1 = At;
   Pt1 = Pt;

   %/** Forecasting **/
   alphahat = TT*At1;
   Phat     = TT*Pt1*TT' + RR*QQ*RR';
   Phat     = .5*(Phat+Phat');

   yhat     = ZZ*alphahat + DD;
   nut      = YY(t,:) - yhat';
   Ft       = ZZ*Phat*ZZ' + HH + ZZ*RR*VV + (ZZ*RR*VV)';
   Ft       = .5*(Ft+Ft');
   
   %/** Updating **/
   Fti = inv(Ft);
   At = alphahat + Phat*ZZ'*Fti*nut';
   Pt = Phat - (Phat*ZZ'+RR*VV)*Fti*(Phat*ZZ'+RR*VV)';
   
end

drws = randn(steps,neps)*chol(QQ);

smat = zeros(ns,steps);

for j = 1:steps
    
    smat(:,j) = TT * At + RR * drws(j,:)';
    
    At = smat(:,j);
    
end
    
           



